# Generated by Django 2.2.23 on 2021-09-15 10:44

from django.db import migrations, models


def batch_qs(qs, batch_size=10):
    total = qs.count()
    for start in range(0, total, batch_size):
        yield qs[start:start+batch_size]


def set_file_size(apps, schema_editor):
    OcrModel = apps.get_model('core', 'OcrModel')

    for ocr_models in batch_qs(OcrModel.objects.all()):
        for ocr_model in ocr_models:
            try:
                ocr_model.file_size = ocr_model.file.size
            except (ValueError, FileNotFoundError) as e:
                print(f"Couldn't update file_size field on {ocr_model.id}, the file wasn't found: {e}")

        OcrModel.objects.bulk_update(ocr_models, ['file_size'])


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0054_documentpart_image_size'),
    ]

    operations = [
        migrations.AddField(
            model_name='ocrmodel',
            name='file_size',
            field=models.BigIntegerField(default=0),
        ),
        migrations.RunPython(
            set_file_size,
            reverse_code=migrations.RunPython.noop,
        ),
        migrations.AlterField(
            model_name='ocrmodel',
            name='file_size',
            field=models.BigIntegerField(),
        ),
    ]
